/*
 * Transition-RRT.hpp
 *
 *  Created on: Nov 20, 2012
 *      Author: ddevaurs
 */

#ifndef _TRANSITION_RRT_HPP_
#define _TRANSITION_RRT_HPP_

#include "planner/Diffusion/RRT.hpp"


/**
 * Expansion procedure of the Transition-based RRT (T-RRT) algorithm.
 */
class TransitionExpansion: public RRTExpansion
{
public:
    //! Constructor.
	TransitionExpansion(Graph * graph, Node * goal);

    //! Destructor.
    virtual ~TransitionExpansion() {};

    /**
     * Attempt to link two given nodes (i.e. two components, when relevant).
     * @return TRUE if the junction is successful.
     */
    bool attemptJunction(Node * fromNode, Node * toNode);

private:
    /**
     * T-RRT Extend.
     * @param fromNode: node from which the expansion is performed
     * @param directionConf: configuration toward which the expansion is going
     * @param directionNode: node toward which the expansion is going (when relevant)
     * @return the number of created nodes
     */
    unsigned extend(Node * fromNode, confPtr_t directionConf, Node * directionNode);

    /**
     * T-RRT Connect.
     * @param fromNode: node from which the expansion is performed
     * @param directionConf: configuration toward which the expansion is going
     * @param directionNode: node toward which the expansion is going (when relevant)
     * @return the number of created nodes
     */
    unsigned connect(Node * fromNode, confPtr_t directionConf, Node * directionNode);

    /**
     * Transition test between fromConf and toConf, with temperature tuning.
     * @return TRUE if the transition is accepted
     */
    bool transitionTest(confPtr_t fromConf, confPtr_t toConf);

    //! Compare the given cost to the current lowest and highest costs, and update them if necessary.
    void updateCostRange(double cost) {
        if (cost > highestCost) highestCost = cost;
        if (cost < lowestCost) lowestCost = cost;
    }

private:
    //! temperature parameter of the T-RRT
    double temperature;

    //! cost of the highest-cost node in the tree
    double highestCost;

    //! cost of the lowest-cost node in the tree
    double lowestCost;
};


/**
 * The Transition-based RRT (T-RRT) algorithm.
 */
class TransitionRRT: public RRT
{
public:
    //! Constructor.
    TransitionRRT(Robot * robot, Graph * graph) : RRT(robot, graph) {
        std::cout << "Construct the T-RRT" << std::endl;
    }

    //! Destructor.
    virtual ~TransitionRRT() {};

    /**
     * Initialize the T-RRT.
     * @return the number of nodes added to the graph
     */
    unsigned init();
};

#endif
